@@ -1,15 +1,13 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from trytond.model import fields
 from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
 from trytond.pyson import Eval
 
 __all__ = ['Sale', 'SaleLine']
-__metaclass__ = PoolMeta
 
 
-class Sale:
+class Sale(metaclass=PoolMeta):
     __name__ = 'sale.sale'
 
     def is_done(self):
@@ -21,9 +19,13 @@ class Sale:
         return done
 
     def create_shipment(self, shipment_type):
-        shipments = super(Sale, self).create_shipment(shipment_type)
         if shipment_type == 'out':
+            # purchase requests must be created before shipments to get
+            # information about requests during the shipments creation
+            # like the supplier
             self.create_purchase_requests()
+            self.create_move_from_purchase_requests()
+        shipments = super(Sale, self).create_shipment(shipment_type)
         return shipments
 
     def create_purchase_requests(self):
@@ -37,8 +39,25 @@ class Sale:
             line.purchase_request = request
             line.save()
 
+    def create_move_from_purchase_requests(self):
+        'Set to draft move linked to purchase requests'
+        pool = Pool()
+        Move = pool.get('stock.move')
+        ShipmentOut = pool.get('stock.shipment.out')
 
-class SaleLine:
+        moves = []
+        for line in self.lines:
+            if line.purchase_request_state in ['purchased', 'cancel']:
+                for move in line.moves:
+                    if move.state == 'staging':
+                        moves.append(move)
+        Move.draft(moves)
+        shipments = {m.shipment for m in moves
+            if isinstance(m.shipment, ShipmentOut)}
+        ShipmentOut.wait(shipments)
+
+
+class SaleLine(metaclass=PoolMeta):
     __name__ = 'sale.line'
 
     purchase_request = fields.Many2One('purchase.request', 'Purchase Request',
@@ -53,28 +72,25 @@ class SaleLine:
                 'invisible': ~Eval('purchase_request_state'),
                 }), 'get_purchase_request_state')
 
-    @classmethod
-    def get_purchase_request_state(cls, lines, name):
-        states = dict((l.id, '') for l in lines)
-        with Transaction().set_user(0, set_context=True):
-            for line in cls.browse(states.keys()):
-                if line.purchase_request is not None:
-                    states[line.id] = 'requested'
-                    purchase_line = line.purchase_request.purchase_line
-                    if purchase_line is not None:
-                        purchase = purchase_line.purchase
-                        if purchase.state == 'cancel':
-                            states[line.id] = 'cancel'
-                        elif purchase.state in ('confirmed', 'done'):
-                            states[line.id] = 'purchased'
-        return states
+    def get_purchase_request_state(self, name):
+        if self.purchase_request is not None:
+            purchase_line = self.purchase_request.purchase_line
+            if purchase_line is not None:
+                purchase = purchase_line.purchase
+                if purchase.state == 'cancel':
+                    return 'cancel'
+                elif purchase.state in ('processing', 'done'):
+                    return 'purchased'
+            return 'requested'
+        return ''
 
     @classmethod
     def copy(cls, lines, default=None):
         if default is None:
             default = {}
-        default = default.copy()
-        default['purchase_request'] = None
+        else:
+            default = default.copy()
+        default.setdefault('purchase_request', None)
         return super(SaleLine, cls).copy(lines, default=default)
 
     @property
@@ -84,17 +100,18 @@ class SaleLine:
                 or not self.product
                 or self.quantity <= 0
                 or not self.product.purchasable
-                or any(m.state != 'cancel' for m in self.moves)):
+                or any(m.state not in ['staging', 'cancel'] for m in self.moves)):
             return False
         return self.product.supply_on_sale
 
     def get_move(self, shipment_type):
         move = super(SaleLine, self).get_move(shipment_type)
-        if (shipment_type == 'out'
+        if (move
+                and shipment_type == 'out'
                 and (self.supply_on_sale
                     or self.purchase_request)):
             if self.purchase_request_state in ('', 'requested'):
-                return
+                move.state = 'staging'
         return move
 
     def get_purchase_request(self):
@@ -106,25 +123,36 @@ class SaleLine:
         if not self.supply_on_sale or self.purchase_request:
             return
 
+        # Ensure to create the request for the maximum paid
+        if self.sale.shipment_method == 'invoice':
+            invoice_skips = (set(self.sale.invoices_ignored)
+                | set(self.sale.invoices_recreated))
+            invoice_lines = [l for l in self.invoice_lines
+                if l.invoice not in invoice_skips]
+            if (not invoice_lines
+                    or any((not l.invoice) or l.invoice.state != 'paid'
+                        for l in invoice_lines)):
+                return
+
         product = self.product
         supplier, purchase_date = Request.find_best_supplier(product,
-            self.delivery_date)
+            self.shipping_date)
         uom = product.purchase_uom or product.default_uom
-        quantity = Uom.compute_qty(self.unit, self.quantity, uom)
-        with Transaction().set_user(0, set_context=True):
-            return Request(
-                product=product,
-                party=supplier,
-                quantity=quantity,
-                uom=uom,
-                computed_quantity=quantity,
-                computed_uom=uom,
-                purchase_date=purchase_date,
-                supply_date=self.delivery_date,
-                company=self.sale.company,
-                warehouse=self.warehouse,
-                origin=self.sale,
-                )
+        quantity = self._get_move_quantity('out')
+        quantity = Uom.compute_qty(self.unit, quantity, uom)
+        return Request(
+            product=product,
+            party=supplier,
+            quantity=quantity,
+            uom=uom,
+            computed_quantity=quantity,
+            computed_uom=uom,
+            purchase_date=purchase_date,
+            supply_date=self.shipping_date,
+            company=self.sale.company,
+            warehouse=self.warehouse,
+            origin=self.sale,
+            )
 
     def assign_supplied(self, location_quantities):
         '''
@@ -148,8 +176,7 @@ class SaleLine:
             if move.state != 'draft':
                 continue
             location_qties_converted = {}
-            for location_id, quantity in (
-                    location_quantities.iteritems()):
+            for location_id, quantity in location_quantities.items():
                 location_qties_converted[location_id] = (
                     Uom.compute_qty(move.product.default_uom,
                         quantity, move.uom, round=False))
